//  -*- C++ -*-

/*
 * Copyright (C) 1991   Silicon Graphics, Inc.
 *
 _______________________________________________________________________
 ______________  S I L I C O N   G R A P H I C S   I N C .  ____________
 |
 |   $Revision: 1.1.1.1 $
 |
 |   Description:
 |	Callback list - a list of callback functions and associated data,
 |   with member functions to add, remove, and invoke callbacks.
 |	
 |   Classes:	SoXtFloatCallbackList
 |
 |   Author(s):	David Mott
 |
 ______________  S I L I C O N   G R A P H I C S   I N C .  ____________
 _______________________________________________________________________
 */

#ifndef HEPVis_SoXtFloatCallbackList_h
#define HEPVis_SoXtFloatCallbackList_h

#include <Inventor/SbPList.h>

//////////////////////////////////////////////////////////////////////////////
//
//  Class: SoXtFloatCallbackList
//
//////////////////////////////////////////////////////////////////////////////

// Callback functions that are registered with this class should
// be cast to this type.
typedef void SoXtFloatCallbackListCB(void *userData, float callbackData);

// C-api: prefix=SoFCBList
class SoXtFloatCallbackList {
    
  public:

    SoXtFloatCallbackList();
    ~SoXtFloatCallbackList();
    
    //
    // Managing callback functions
    // At callback time, f will be called with userData as the first
    // parameter, and callback specific data as the second parameter.
    // e.g. (*f)(userData, callbackData);

    // C-api: name=addCB
    void    addCallback(SoXtFloatCallbackListCB *f, void *userData = NULL);
    // C-api: name=removeCB
    void    removeCallback(SoXtFloatCallbackListCB *f, void *userData = NULL);
    
    // C-api: name=clearCB
    void    clearCallbacks()			    { list.truncate(0); }
    // C-api: name=getNumCB
    int	    getNumCallbacks() const		    { return list.getLength(); }

    // C-api: name=invokeCB
    void    invokeCallbacks(float callbackData);

  private:
  
    // callbackList holds a list of functions and user data
    SbPList 	list;
};

#endif
