#ifndef Lib_Parameters_h
#define Lib_Parameters_h

#include <list>
#include <string>
#include <vector>

namespace Lib {

template <class K,class V>
class Parameter {
public:
  Parameter(
   int aCategory
  ,const K& aKey
  ,const std::string& aLabel
  ,const V& aValue
  ,const V& aDefault
  ,const V& aAlternative
  )
  :fCategory(aCategory)
  ,fKey(aKey)
  ,fLabel(aLabel)
  ,fValue(aValue)
  ,fDefault(aDefault)
  ,fAlternative(aAlternative)
  ,fTouched(false){
  }
  virtual ~Parameter(){}
  Parameter(const Parameter& aFrom)
  :fCategory(aFrom.fCategory)
  ,fKey(aFrom.fKey)
  ,fLabel(aFrom.fLabel)
  ,fValue(aFrom.fValue)
  ,fDefault(aFrom.fDefault)
  ,fAlternative(aFrom.fAlternative)
  ,fTouched(false){
  }
  Parameter& operator =(const Parameter& aFrom){
    fCategory = aFrom.fCategory;
    fKey = aFrom.fKey;
    fLabel = aFrom.fLabel;
    fValue = aFrom.fValue;
    fDefault = aFrom.fDefault;
    fAlternative = aFrom.fAlternative;
    fTouched = false;
    return *this;
  }
  const int category() const { return fCategory;}
  const bool touched() const { return fTouched;}
  const K& key() const { return fKey;}
  const std::string& label() const { return fLabel;}
  const V& value() const { return fValue;}
  const V& defaultValue() const { return fDefault;}
  const V& alternative() const { return fAlternative;}
public:
  int fCategory;
  K fKey;
  std::string fLabel;
  V fValue;
  V fDefault;
  V fAlternative;
  bool fTouched;
};


template <class K,class V>
class Parameters {
public:
  typedef std::list< Parameter<K,V> > List;
  typedef typename List::iterator Iterator;
  typedef typename List::const_iterator ConstIterator;

  void add(int aCategory,const K& aKey,const std::string& aLabel,const V& aValue,const V& aDefault,const V& aAlternative) {
    fList.push_back
      (Parameter<K,V>(aCategory,aKey,aLabel,aValue,aDefault,aAlternative));
  }

  void remove(const K& aKey) {
    Iterator it;
    for(it=fList.begin();it!=fList.end();++it) {
      if(aKey==(*it).fKey) {
        //fList.remove(*it); //Need a operator==(Parameter,Parameter)
        fList.erase(it);
        return;
      }
    } 
  }
  void setValue(const K& aKey,V aValue) {
    Iterator it;
    for(it=fList.begin();it!=fList.end();++it) {
      if(aKey==(*it).fKey) {
        (*it).fValue = aValue;
        (*it).fTouched = true;
        return;
      }
    } 
  }
  void setAllToDefault() {
    Iterator it;
    for(it=fList.begin();it!=fList.end();++it) {
      (*it).fValue = (*it).fDefault;
      (*it).fTouched = false;
    }
  }
  void setCategoryToDefault(int aCategory) {
    Iterator it;
    for(it=fList.begin();it!=fList.end();++it) {
      if((*it).fCategory==aCategory) {
        (*it).fValue = (*it).fDefault;
      }
    }
  }
  void setToDefault(const K& aKey) {
    Iterator it;
    for(it=fList.begin();it!=fList.end();++it) {
      if(aKey==(*it).fKey) {
        (*it).fValue = (*it).fDefault;
      }
    }
  }
  std::string label(const K& aKey) const {
    ConstIterator it;
    for(it=fList.begin();it!=fList.end();++it) {
      if(aKey==(*it).fKey) {
        return (*it).fLabel;
      }
    }
    return "";
  }
  bool category(const K& aKey,int& aCategory) const {
    ConstIterator it;
    for(it=fList.begin();it!=fList.end();++it) {
      if(aKey==(*it).fKey) {
        aCategory = (*it).fCategory;
        return true;
      }
    }
    return false;
  }
  bool touched(const K& aKey,bool& a_touched) const {
    ConstIterator it;
    for(it=fList.begin();it!=fList.end();++it) {
      if(aKey==(*it).fKey) {
        a_touched = (*it).fTouched;
        return true;
      }
    }
    a_touched = false;
    return false;
  }
  V value(const K& aKey) const {
    ConstIterator it;
    for(it=fList.begin();it!=fList.end();++it) {
      if(aKey==(*it).fKey) {
        return (*it).fValue;
      }
    }
    return fValueNotFound;
  }
  V defaultValue(const K& aKey) const {
    ConstIterator it;
    for(it=fList.begin();it!=fList.end();++it) {
      if(aKey==(*it).fKey) {
        return (*it).fDefault;
      }
    }
    return fValueNotFound;
  }
  const K& key(V aValue) const {
    ConstIterator it;
    for(it=fList.begin();it!=fList.end();++it) {
      if(aValue==(*it).fValue) { //Need operator==(V,V)
        return (*it).fKey;
      }
    }
    return fKeyNotFound;
  }
  const K& keyBoolean(V aValue) const {
    ConstIterator it;
    for(it=fList.begin();it!=fList.end();++it) {
      if( (aValue==(*it).fDefault) || 
          (aValue==(*it).fAlternative) ) { //Need operator==(V,V)
        return (*it).fKey;
      }
    }
    return fKeyNotFound;
  }
  bool isKey(const K& aKey) const {
    ConstIterator it;
    for(it=fList.begin();it!=fList.end();++it) {
      if(aKey==(*it).fKey) {
        return true;
      }
    }
    return false;
  }
  const List& list() const { return fList;}
  const K& keyNotFound() const { return fKeyNotFound;}
  V valueNotFound() const { return fValueNotFound;}
  std::vector<K> keys() const {
    std::vector<K> ks;
    ConstIterator it;
    for(it=fList.begin();it!=fList.end();++it) {
      ks.push_back((*it).fKey);
    }
    return ks;
  }
public:
  Parameters(const K& aKeyNotFound,V aValueNotFound)
  :fKeyNotFound(aKeyNotFound),fValueNotFound(aValueNotFound){}
  virtual ~Parameters(){}
protected:
  K fKeyNotFound;
  V fValueNotFound;
  List fList;
};

}

#endif



